;----------------------------------------------------------------------------
; Pactor demodulator for use with EVM
; (C) Johan Forrer, KC7WW, 1997. All rights reserved.
;                       forrerj@peak.org
;
; Heterodyne demodulator method as described by Frerking;p ???.
; 
; This code is for educational purposes and is provided for the
; enjoyment by radio amateurs. Commercial or military use require
; the written consent of the author(s).
;
; This software is provided "as is". No guarantees of liabilities of 
; whatever nature is acceptible. 
;----------------------------------------------------------------------------

	PAGE 132
	opt mu		; we want a memory map

;----------------------------------------------------------------------------
; NOTE NOTE NOTE: set the DSPCARD equate for EVM/DSPCARD: NOTE NOTE NOTE
;----------------------------------------------------------------------------
DSPCARD equ 0
;----------------------------------------------------------------------------

	nolist
	include 'ttycod.asm'
	list

;----------------------------------------------------------------------------
; A few notes on timing accuracy and how to fix it:
;----------------------------------------------------------------------------
; AMTOR/SITOR/Pactor needs a timing accuracy/stability of at least 20ppm.
; This version of the interface program derives all timing from the
; CODEC's 24.576 MHz crystal and further allows accurate tuning of the 
; clock:
;
; Some calculations:
; 	2457600/2560 = 9600 SPS for the CODEC sampling rate.
;	9600/9.6 = 0.104167*9600 = 1000 Hz timing
;
; Using a good counter, I found my 24.576000 MHz signal at 24.576019 MHz,
; i.e., +0.77ppm which appears to be within the crystal's tolerance.
; This puts the timing at carrier at 1000.00077311 Hz.  
;
; The trick is then to adjust the fixed constant (0.104167) such that we 
; come out at exactly at 1000 Hz. In my case, the corrected constant
; is 0.104166586134.
;
; Using 32-bit arithmetic the nominal factor should be
; Factor = 1000/9600*65536*65536 = $1AAA AAAB.
; For the correct rate, the adjusted factor now is,
; Factor = 0.104167*65536*65536 = $1AAA A951
;----------------------------------------------------------------------------
CLFACH		equ	$1AAA	; Nominal
CLFACL		equ	$AAAB
;------------------------------------------
;CLFACH		equ	$1AAA	; Adjusted for 19 Hz high crystal
;CLFACL		equ	$A951
;----------------------------------------------------------------------------

SampleFreq	equ 	9600.0

; These frequencies can be set to whatever sounds reasonable
CenterFreq 	equ 	1360.0  ; Low tone set
;CenterFreq	 equ 	2210.0  ; High tone set
AliasFilterLen  equ   	64   		;Anti-alias FIR filter length
AliasLowFreq    equ  	CenterFreq-150 	;[Hz] lower/upper edges of input/output filters
AliasUppFreq    equ  	CenterFreq+150 	;[Hz]

SymbolRate      equ 	200   		;[Hz]  bits/second
SymbolLen       equ 	@cvi(SampleFreq/SymbolRate+0.5) ;[samples @ 9600 SPS ]
SymbolLenLog2   equ 	@cvi(@log(SymbolLen)/@log(2)+0.5)

LowToneFreq   	equ 	CenterFreq-100 	;[Hz] Lower  FSK tone frequency
HighToneFreq  	equ 	CenterFreq+100 	;[Hz] Higher FSK tone frequency
CarDev	      	equ  	100.0	

BitFilterLen   	equ 	64     	; Post-detection filtering for Rx

htaps		equ    	384 	; #taps in trace history

;----------------------------------------------------------------------------
; Specify here whether expanded EVM LED display is used.
;----------------------------------------------------------------------------
 IF (DSPCARD==1)
LED_DISPLAY 	equ 	0
 else
LED_DISPLAY 	equ 	1	; For expanded hardware, use LED array
 endif
;----------------------------------------------------------------------------
; Further assembly directives.
;----------------------------------------------------------------------------
YES		equ	1
NO		equ	0

LEFT 		equ	1
RIGHT		equ	2

usesig		equ	LEFT	; ***Use LEFT audio input***

;----------------------------------------------------------------------------
; Configure audio I/O
;----------------------------------------------------------------------------
TONE_OUTPUT     EQU     HEADPHONE_EN+LINEOUT_EN+(0*LEFT_ATTN)+(0*RIGHT_ATTN)   

 if DSPCARD
TONE_INPUT      EQU     (15*MONITOR_ATTN)+(10*RIGHT_GAIN)+(10*LEFT_GAIN)
OVR_CLR         EQU     (15*MONITOR_ATTN)+(10*RIGHT_GAIN)+(10*LEFT_GAIN)+$200000
 else
TONE_INPUT      EQU     MIC_IN_SELECT+(15*MONITOR_ATTN)+(10*RIGHT_GAIN)+(10*LEFT_GAIN)
OVR_CLR         EQU     MIC_IN_SELECT+(15*MONITOR_ATTN)+(10*RIGHT_GAIN)+(10*LEFT_GAIN)+$200000
 endif

REVERSED 	equ 0
 IF (REVERSED==1)
TONE1		equ    	1275.0	; AFSK tones to be generated
TONE2		equ	1445.0
 else
TONE1		equ	1445.0 	; normal
TONE2		equ    	1275.0
 ENDIF

;SECONDS	equ	600	; 1 second tics for a 1200 Hz clock
SECONDS		equ	500	;         ..... for a 1000 Hz clock

;----------------------------------------------------------------------------
; macro for red LED handling
cmdLED	macro	mode
	b\mode	#13,x:PBD
	endm

; macro for green LED handling
gnLED	macro	mode
	b\mode	#14,x:PBD
	endm

; macro for Red Mark LED handling
markLED	macro	mode
	b\mode	#1,x:PBD
	endm

; macro for Red Space LED handling
spaceLED	macro	mode
	b\mode	#2,x:PBD
	endm

; macro for PTT handling
PTTleft	macro	mode
	b\mode	#0,x:PBD
	endm

; macro for DCD handling
DCD	macro	mode
	b\mode	#11,x:PBD
	endm

; macro for CTS handling
CTS	macro	mode
	b\mode	#10,x:PBD
	endm
;----------------------------------------------------------------------------
	org     p:
main

; initialize port B
	movep	    #$0000,x:PBC	    ; port B as general purpose port
;;	movep	    #$60ff,x:PBADDR	    ; PB0-PB7,PB13 and PB14 as outputs
	movep	    #$6Cff,x:PBADDR	    ; PB0-PB7,PB10,11,13,14 outputs
	movep	    #$0000,x:PBD

	movep	#$1,x:TCSR		; timer mode 0, GPIO
;;	movep	#>10000-1,x:TCR		; 40Mhz/2/1000/2 - 1
	movep	#>10000-1,x:TCR		; 48Mhz/2/1200/2 - 1

	move	#>SECONDS,a		; for 1 second counts
	move	a,x:tcount		; clear tcount

; Initialize SCI port
; Use internal clocks
	
	movep	#>$0302,x:SCR		; 1 start, 1 stop, 8 data (10 bits) 
					; with no interrupts enabled
;	movep	#>$0004,x:SCCR		; CD=(40 MHz/bps/64) - 1 = 4 (115.2)
	movep	#>$0020,x:SCCR		; 19200
					; 57600 =>  10 
					; 28800 =>  21 
					; 14400 =>  42 
					;  9600 =>  64 
					;  7200 =>  86 
					;  4800 => 129 
					;  3600 => 173 
					;  2400 => 259 
					;  1200 => 520 

    	bset    #0,x:PCC		; allocate TxD and RxD pins
    	bset    #1,x:PCC

;;	movep	#>$3,x:TCSR		; Enable Timer interrupts	

        ori #%00000100,omr      	; enable the sine ROM table
                                	; this is for the FreqSynth routine

	movep   #$fb00,x:CRB            ; Enable the SSI interrupts.
	movep	#$02301b,x:IPR		; IRQA/IRQB/SSI are level 3 interrupts.
  					; and are LEVEL sensitive

;
; Program headphone,lineout enabled, speaker off, attn=0
; The codec ISR will write these values on each CODEC cycle.
; We may wish to update the setting on the fly, if we wish, as they are
; in on-chip ram anyways.
;
	move    #TONE_OUTPUT,y0         ; headphones, line out, 
					; mute spkr, no attn.
	move    y0,x:<TX_BUFF_BASE+2
	move    #TONE_INPUT,y0          ; no input gain, monitor mute
	move    y0,x:<TX_BUFF_BASE+3

;---------------------------------------------------------------------------
; Initialize buffer pointer
;---------------------------------------------------------------------------
	move	#Buffer,r0		; data_in_ptr = input buffer pointer
	move	r0,x:data_in_ptr

	move	#BitFilter,r0		; bitfilter_ptr = data LP
	move	r0,x:bitfilter_ptr

	move	#LowToneFiltI,r0	; LowToneI_ptr
	move	r0,x:lowtoneI_ptr
	move	#LowToneFiltQ,r0	; LowToneQ_ptr
	move	r0,x:lowtoneQ_ptr

	move	#HighToneFiltI,r0	; HighToneI_ptr
	move	r0,x:hightoneI_ptr
	move	#HighToneFiltQ,r0	; HighToneQ_ptr
	move	r0,x:hightoneQ_ptr

	move	#history,r0	; trace buffer
	move	r0,x:hist_ptr

; Initialize sine/cosine variables
	move	#>2.0*LowToneFreq/SampleFreq,a
	move	a,x:LowToneIncr

	move	#>2.0*HighToneFreq/SampleFreq,a
	move	a,x:HighToneIncr

	move	#>2.0*TONE1/SampleFreq,a  	; Build AFSK tone library
	move	a,x:Tone1
	move	#>2.0*TONE2/SampleFreq,a
	move	a,x:Tone2

	clr	a
	move	a,x:LowTonePhase
	move	a,x:HighTonePhase
	move	a,x:AFSKPhase 			; tone generator
	move	a,x:decimate
	move	a,x:m_peak
	move	a,x:s_peak

	move	a,x:ClaccH		; Clear out clock pulse accumulator 
	move	a,x:ClaccL

	move	a,x:TxState		; clear TxState
	move	a,x:PTTFlag		; start with PTT de-asserted

; Clear any pending data in SCI
	move	x:SSR,a
	move	x:SRX,a

	andi    #$FC,mr         ; enable interrupts

;--------------- Main Loop --------------------------------------------------
loop_1 
	jset    #2,x:SSISR,*            ; Wait for frame sync to pass.
	jclr    #2,x:SSISR,*            ; Wait for frame sync.

;----------------------------------------------------------------------------
; Read input signal.
; Note: Input may be overdriven, which case results will saturate.
; Detect overload condition from Slot 7, bit 5 (RX_BUFF_BASE+3 = $20)
; or when read left justified into a 24-bit 20xxxx.
; The bit is reset by writing zero.
;----------------------------------------------------------------------------
	move    x:RX_BUFF_BASE,a      	; Load the left channel input.
	move	a1,x:LEFT_sig		; data appears left justified

;-------TEST-------------------
;	move    a,x:TX_BUFF_BASE        ; Put value in left channel tx.
;	move    a,x:TX_BUFF_BASE+1      ; Put value in right channel tx.  
;-------TEST-------------------

	move    x:RX_BUFF_BASE+1,b     ; Load the right channel input.
	move	b1,x:RIGHT_sig

;----------------------------------------------------------------------------
; Read status of switches/RTS/DTR
;----------------------------------------------------------------------------
	movep	x:PBD,a

 IF (DSPCARD==1)
	move	#>$300,a		; disable switches
 ENDIF
	move	a1,x:switches

; Test for CODEC overload condition
	move    x:<RX_BUFF_BASE+3,a
	move	#>$200000,x0
	and	x0,a
	jeq	sig_good
	cmdLED	set
	move    #TONE_INPUT,y0		; clear overload bit
	move    y0,x:<TX_BUFF_BASE+3
	jmp	process
sig_good
	cmdLED	clr
	move    #OVR_CLR,y0		; enable overload detection
	move    y0,x:<TX_BUFF_BASE+3
process

;----------------------------------------------------------------------------
; Test for SCI data and set/clear PPT flag if needed
;----------------------------------------------------------------------------
	jclr	#0,x:SSR,no_data
	movep	x:SRX,x1	; get SCI character
				; extract PTT bit
	move	x1,x:TxState	;save state
	jset	#0,x1,set_PTT	; PTT request?
	clr	a		; no, clear PTTFlag	
	jmp	set_PTTFlag
set_PTT:	
	move	#>1,a		; yes, set PTTFlag
set_PTTFlag:
	move	a1,x:PTTFlag
no_data:	

;----------------------------------------------------------------------------
; When PTTFlag is asserted, transmit audio
;----------------------------------------------------------------------------
	move	x:PTTFlag,a
	tst	a
	jeq	Recvr

	PTTleft	set			; ensure PTT is asserted
	move	x:AFSKOut,a		; get internal AFSK signal
	asr	a
	asr	a
	move    a1,x:<TX_BUFF_BASE        ; Put value in left channel tx.
	move    a1,x:<TX_BUFF_BASE+1      ; Put value in right channel tx.  
	
	jmp	Transmit

;---------------------------------------------------------------------------
; Output input bpf to headphones
;----------------------------------------------------------------------------
Recvr
	PTTleft	clr			; ensure PTT is released

	move	x:TmpI,a
;	move	x:TmpQ,a
	.loop #8		; CODEC needs it left justified
	asl	a
	.endl

	move    a1,x:<TX_BUFF_BASE        ; Put value in left channel tx.
	move    a1,x:<TX_BUFF_BASE+1      ; Put value in right channel tx.  

;----------------------------------------------------------------------------
; Prepare data pointers to receive sampled data
;----------------------------------------------------------------------------
	move	x:data_in_ptr,r0
        move    #AliasFilterLen-1,m0     ; mod(ntaps)

 IF (usesig==LEFT) 			; Select which input to use
	move	x:LEFT_sig,a		; data appears left justified
	move	a1,x0
	mpy	x0,#8,a
	move	a1,x:(r0)+		; save in circular buffer
 ENDIF

 IF (usesig==RIGHT)
	move	x:RIGHT_sig,a		; data appears left justified
	move	a1,x0
	mpy	x0,#8,a
	move	a1,x:(r0)+		; save in circular buffer
 ENDIF

	move	r0,x:data_in_ptr	; save updated pointer
	
;-----------------------------------------------------------------------------
; Perform input filtering.
; Two bandpass filters, 90 apart, are used to create a 
; filtered I/Q stream. 
;-----------------------------------------------------------------------------
	move	x:data_in_ptr,r0
        move    #AliasFilterLen-1,m0	; mod(ntaps)
	move 	#AliasFilterInpI,r4 	; r4 points to input filter coefficeints
	move	#>-1,m4			; linear addressing
	nop
	clr a X:(r0)+,x0 Y:(r4)+,y0
	.loop #AliasFilterLen-1
            mac x0,y0,a X:(r0)+,x0  Y:(r4)+,y0
	.endl
	macr x0,y0,a
	move a,x:TmpI
 
	move	x:data_in_ptr,r0
        move    #AliasFilterLen-1,m0     ; mod(ntaps)
	move #AliasFilterInpQ,r4
	nop
	clr b X:(r0),x0 Y:(r4)+,y0
	.loop #AliasFilterLen-1
            mac x0,y0,b X:(r0)+,x0  Y:(r4)+,y0
	.endl
	macr x0,y0,b 			; a,b = Filtered sample (I/Q)
	move b,X:TmpQ

;-------------------------------------------------------------------------------
; Process the I/Q sample in X:<RxTmpI/Q
; Extract LSB.
;-------------------------------------------------------------------------------
        move 	X:HighTonePhase,x0  	; Advance high tone phase
        move 	X:HighToneIncr,a
        add 	x0,a
        move 	a1,X:HighTonePhase
        jsr 	FreqSyn                 ; Compute sine/cosines
;	move	a,x:temp		; save cosine
;	move	b,x:temp		; save sine

        move a,y1             		; mix I/Q with the tone's carrier
        move b,y0
        move X:TmpI,x1
        mpy x1,y1,a X:TmpQ,x0		; I.cos
        macr x0,y0,a			; + Q.sin

        mpy x1,y0,b     		; I.sin
        macr -x0,y1,b			; - Q.sin

	move	x:hightoneI_ptr,r0	; Save results in symbol buffers
        move    #SymbolLen-1,m0		; mod m0
	nop
	move	a,x:(r0)+
	move	r0,x:hightoneI_ptr

	move	x:hightoneQ_ptr,r0
	nop	
	move	b,x:(r0)+
	move	r0,x:hightoneQ_ptr

;--------------------------------
        move 	X:LowTonePhase,x0  	; Advance low tone phase
        move 	X:LowToneIncr,a
        add 	x0,a
        move 	a1,X:LowTonePhase
        jsr 	FreqSyn                 ; Compute sine/cosines
;	move	a,x:temp		; save cosine
;	move	b,x:temp		; save sine

        move a,y1             		; mix I/Q with the tone's carrier
        move b,y0
        move X:TmpI,x1
        mpy x1,y1,a X:TmpQ,x0		; I.cos
        macr x0,y0,a			; + Q.sin

        mpy x1,y0,b     		; I.sin
        macr -x0,y1,b			; - Q.sin

	move	x:lowtoneI_ptr,r0	; Save results in symbol buffers
        move    #SymbolLen-1,m0		; mod m0
	nop
	move	a,x:(r0)+
	move	r0,x:lowtoneI_ptr

	move	x:lowtoneQ_ptr,r0
	nop
	move	b,x:(r0)+
	move	r0,x:lowtoneQ_ptr

;-----------------------------------------------------------------------------
;  Integrate & Dump filters
;-----------------------------------------------------------------------------
					; Filter low tone I
	move	x:lowtoneI_ptr,r0
	nop
	move x:(r0)+,a           	; sum up the low tone filter's tap
      	move x:(r0)+,x1            	; we are doing a boxcar filter
      	.loop #SymbolLen-2		; but we could put here any shape
         add x1,a X:(r0)+,x1
      	.endl
      	add x1,a

	move	x:lowtoneQ_ptr,r0
	nop
	move x:(r0)+,b			; Filter low tone Q
      	move x:(r0)+,x1
      	.loop #SymbolLen-2
         add x1,b X:(r0)+,x1
      	.endl
      	add x1,b

      	.loop #SymbolLenLog2-1
         asr a
         asr b
	.endl

        move a,y0               	;save low tone filter output in y0/1
        move b,y1
	move	a,x:LoI
	move	b,x:LoQ

;-----------------------------------------------------------------------------
					; Filter high tone I
	move	x:hightoneI_ptr,r0
	nop
	move x:(r0)+,a           	; sum up the high tone filter's tap
      	move x:(r0)+,x1            	; we are doing a boxcar filter
      	.loop #SymbolLen-2		; but we could put here any shape
         add x1,a X:(r0)+,x1
      	.endl
      	add x1,a

	move	x:hightoneQ_ptr,r0
	nop
	move x:(r0)+,b			; Filter high tone Q
      	move x:(r0)+,x1
      	.loop #SymbolLen-2
         add x1,b X:(r0)+,x1
      	.endl
      	add x1,b

      	.loop #SymbolLenLog2-1
         asr a
         asr b
	.endl

	move	a,x:HiI
	move	b,x:HiQ

;----------------------------------------------------------------------------
; Compute respective filter's magnitudes
;----------------------------------------------------------------------------
	move	a,x0
	move	b,x1
        mpy x0,x0,b x1,x0
        mac x0,x0,b       	;b = High tone's power
	asr	b
	asr	b		; Scale down
	asr	b
	move	b0,b
	move b1,x:HiAmpl

        mpy y0,y0,a y1,y0
        mac y0,y0,a       	;a = low tone's power
	asr	a
	asr	a		; Scale down
	asr	a
	move	a0,a
	move a1,x:LoAmpl


;---------------------------------------------------------------------------
; Perform post detection data filtering
; Sample rate = 9600 SPS
; On completion, pointers is same as in beginning, accum a contains result
;---------------------------------------------------------------------------
	move	x:bitfilter_ptr,r0
        move    #BitFilterLen-1,m0  
	nop
	sub	a,b		; result is power difference 
	move	b,x:(r0)+

        move    #RxBitFilter,r4	; R4 point to filter coefficients
        move    #>-1,m4     	; linear addressing
	nop
	move	r0,x:bitfilter_ptr

        clr     a    x:(r0)+,x0  y:(r4)+,y0     ; priming registers
        .loop	#BitFilterLen-1			; allow interrupts
	mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
	.endl
        macr    x0,y0,a		; final rounding
	move	a,x:dataout

;----------------------------------------------------------------------------
; Slicer  output = 0 when a<0, else output = 1
;----------------------------------------------------------------------------
	move	x:dataout,a		; slice data output
	tst	a	
	jlt	is_space 		; Slicer
	CTS	clr
;;					; Clear output bit
;;	DCD	clr
	markLED	  set
	spaceLED  clr
	jmp     <is_mark
is_space
	CTS	set
;;					; Set output bit
;;	DCD	set
	markLED	  clr
	spaceLED  set
is_mark

;----------------------------------------------------------------------------
; Normal single dot display at data lowpass output
;----------------------------------------------------------------------------
 if (LED_DISPLAY==YES) 
	jsr	LEDS
 endif

	jmp	wrapup

;----------------------------------------------------------------------------
; AFSK generation, update emitted signal phases
;----------------------------------------------------------------------------
Transmit
	move	x:TxState,a		; test tx bit state
	jclr	#1,a,tone1
tone2:
	move	x:Tone2,a
	move	a,x:AFSKIncr
	jmp	tonegen	
tone1:
	move	x:Tone1,a
	move	a,x:AFSKIncr

tonegen:
        move 	X:AFSKPhase,x0  	; get AFSK phase accumulator
        move 	X:AFSKIncr,a    	; advance phase
        add 	x0,a
        move 	a1,X:AFSKPhase
        jsr 	FreqSyn                 ; Compute sine/cosines
					; modifies a,b,x,y,r0,m0,n0
	move	a1,a
	move	a1,x:AFSKOut		; save for later use
	

;----------------------------------------------------------------------------
; Build a trace buffer for debugging purposes
;----------------------------------------------------------------------------
wrapup
	move	x:hist_ptr,r0		; save in trace buffer
	move    #>htaps-1,m0
;;;	move 	x:temp,a
;;;	move 	x:fmdemod,a
;;;	move 	x:FSK,a
	move 	x:dataout,a
;;;	move 	x:fmampl,a
;;;	move	a,x:(r0)+
;;;	neg	a
	move	a,x:(r0)+
	move	r0,x:hist_ptr

;-----------------------------------------------------------------------------
; Generate 1000 Hz timing
; Produce an output pulse upon overflow resulting from the following
; 32-bit addition: ACCUM = ACCUM + 1000/9600*65536*65536 = ACCUM + $1AAA AAAB
;-----------------------------------------------------------------------------
	move	x:ClaccL,a		; add LSBs
	move	#>CLFACL,x0		; correction
	add	x0,a
	move	a,r5			; convert 24-bits to 16 bits
	move	r5,x:ClaccL		; save LSB
	move	#>$FF0000,x0		; perform carry when overflow occurs
	and	x0,a
	jeq	add_msb_nc

	move	x:ClaccH,a		; add MSBs (with carry)
	move	#>CLFACH,x0		; clock factor LSB
	add	x0,a
	move	#>$1,x0			; add carry
	add	x0,a
	move	a,r5			; convert 24-bits to 16 bits
	move	r5,x:ClaccH		; save MSB
	jmp	test_ovfl

add_msb_nc
	move	x:ClaccH,a		; add MSBs (no carry)
	move	#>CLFACH,x0		; clock factor MSB
	add	x0,a
	move	a,r5			; convert 24-bits to 16 bits
	move	r5,x:ClaccH		; save MSB

test_ovfl
	move	#>$FF0000,x0		; produce pulse when overflow occurs
	and	x0,a
	jeq	loop_1			; Loop back.
	jsr	ClockPulse
	jmp	loop_1			; Loop back

;-----------------------------------------------------------------------------
; Routine computes a cosine/sine pair using the sine ROM
; with a second order (linear+quadrature) approximation 
; between table points.
; 	x,y,r0,m0,n0 are modified.
; 	Maximum error is about 0.7E-6
; 	Execution time 4+64+4 clock cycles including "jsr <IQ_lev2" and "rts"
;-----------------------------------------------------------------------------
PI      equ     3.14159265358979323846
FreqSyn                             	; x0 = angle ( -1 = -PI, +1 = +PI)
        ori 	#%00000100,omr      	; enable the sine ROM table
        move 	#>$80,x1   		; shift out 8 most significant bits
        mpy 	x0,x1,a  #>$FF,x0
        move 	x0,m0
        and 	x0,a     #>$100,x0
        or 	x0,a     #<$40,n0
        move 	a1,r0      		; put the 8 most significant bits 
					; into r0 with offset = $100
        move 	a0,y0      		; save the remaining bits in y0
        jclr 	#23,y0,SinTable_lev2
        move 	(r0)+
SinTable_lev2
        move 	Y:(r0+n0),x0       	; x0 = coarse cosine
        move 	Y:(r0),x1          	; x1 = coarse sine
        mpyr 	x1,y0,a  #PI/256.0,y1
        tfr 	x0,a  a,x1
        macr 	-x1,y1,a           	;a = fine cosine
        mpyr 	x0,y0,b  Y:(r0),x1
        tfr 	x1,b  b,x1
        macr 	x1,y1,b  #PI*PI/2.0/65536.0,y1  ;b = fine sine
        mpyr 	y0,y0,a  a,x0
        move 	a,y0
        mpyr 	y0,y1,a
        tfr 	x0,a  a,y1
        macr 	-x0,y1,a  b,x1     	;a = super fine cosine
        macr 	-x1,y1,b           	;b = super fine sine
        andi 	#0,omr      		;disable the sine ROM table
        rts

;----------------------------------------------------------------------------
; Routine to display on LEDS
;----------------------------------------------------------------------------
LEDS_DECAY equ 1000
LEDS					
	move	x:dataout,a
	tst	a  x:m_peak,b
	jge	set_mk

	neg	a x:s_peak,b		; make it a positive number
;;	rep	#4
;;	asr	a			; scale to usable level
					; SPACE
	sub	a,b #>LEDS_DECAY,x0
	jlt	l_0			; new data> peak ?

	move	x:s_peak,a 		; no, decay peak slowly
	sub	x0,a
	jge	l_0
	clr	a
l_0					; new peak found
	move	a,x:s_peak		; register it
					; now process for LED display
	move	#>35,y1
	move	#>$000,x1
	cmp	y1,a #>380,y1
	jlt	disp_led
	move	#>$040,x1
	cmp	y1,a #>4095,y1
	jlt	disp_led
	move	#>$080,x1
	cmp	y1,a #>44102,y1
	jlt	disp_led
	move	#>$100,x1
	cmp	y1,a #>474860,y1
	jlt	disp_led
	move	#>$200,x1
	cmp	y1,a #>5112913,y1
	jlt	disp_led
	move	#>$400,x1
	cmp	y1,a 
	jlt	disp_led
	move	#>$800,x1
	jmp	<disp_led

set_mk
	move 	x:m_peak,b
;;	rep	#4
;;	asr	a			; scale to usable level
	sub	a,b  #>LEDS_DECAY,x0
	jlt	l_00			; new data> peak ?

	move	x:m_peak,a		; no, decay peak slowly
	sub	x0,a
	jge	l_00
	clr	a
l_00					; new peak found
	move	a,x:m_peak		; register it
					; now process for LED display
	move	#>35,y1
	move	#>$000,x1
	cmp	y1,a #>380,y1
	jlt	disp_led
	move	#>$020,x1
	cmp	y1,a #>4095,y1
	jlt	disp_led
	move	#>$010,x1
	cmp	y1,a #>44102,y1
	jlt	disp_led
	move	#>$008,x1
	cmp	y1,a #>474860,y1
	jlt	disp_led
	move	#>$004,x1
	cmp	y1,a #>5112913,y1
	jlt	disp_led
	move	#>$002,x1
	cmp	y1,a
	jlt	disp_led
	move	#>$001,x1		; Maximum

disp_led
	move	x1,a
	move	#$FFFFFF,x0
	eor	x0,a
	move    a1,Y:$8000		; output on LEDs
	rts

;----------------------------------------------------------------------------
; Timer ISR
;----------------------------------------------------------------------------
TIMER:

 IF (DSPCARD==1)
	rti
 ELSE
	move    a0,x:save_a0
	move    a1,x:save_a1
	move    a2,x:save_a2
	move    x0,x:save_x0

	move    x:save_a0,a0
	move    x:save_a1,a1
	move    x:save_a2,a2
	move    x:save_x0,x0
	rti
 ENDIF

;----------------------------------------------------------------------------
; Clock generator, 1000 Hz
;----------------------------------------------------------------------------
ClockPulse

	move	x:tcount,a
	move	#>1,x0
	sub	x0,a
	jne	done		
	gnLED	chg
	move	#>SECONDS,a
done
	move	a1,x:tcount

;----------------------------------------------------------------------------
; Power function to emulate 
; 7 bits+sign for sending over serial line
;----------------------------------------------------------------------------
	move	x:dataout,a
	tst	a
	jge	adset_mk

	neg	a			; make it a positive number
					; SPACE
	move	#>3,y1
	move	#>0,x1
	cmp	y1,a #>35,y1
	jlt	transf
	move	#>-1,x1
	cmp	y1,a #>115,y1
	jlt	transf
	move	#>-2,x1
	cmp	y1,a #>380,y1
	jlt	transf
	move	#>-3,x1
	cmp	y1,a #>1248,y1
	jlt	transf
	move	#>-4,x1
	cmp	y1,a #>4095,y1
	jlt	transf
	move	#>-5,x1
	cmp	y1,a #>13440,y1
	jlt	transf
	move	#>-6,x1
	cmp	y1,a #>44102,y1
	jlt	transf
	move	#>-7,x1
	cmp	y1,a #>144715,y1
	jlt	transf
	move	#>-8,x1
	cmp	y1,a #>474860,y1
	jlt	transf
	move	#>-9,x1
	cmp	y1,a #>1558177,y1
	jlt	transf
	move	#>-10,x1
	cmp	y1,a #>5112913,y1
	jlt	transf
	move	#>-11,x1
	cmp	y1,a 
	jlt	transf
	move	#>-12,x1
	jmp	<transf

adset_mk
	move	#>3,y1
	move	#>0,x1
	cmp	y1,a #>35,y1
	jlt	transf
	move	#>1,x1
	cmp	y1,a #>115,y1
	jlt	transf
	move	#>2,x1
	cmp	y1,a #>380,y1
	jlt	transf
	move	#>3,x1
	cmp	y1,a #>1248,y1
	jlt	transf
	move	#>4,x1
	cmp	y1,a #>4095,y1
	jlt	transf
	move	#>5,x1
	cmp	y1,a #>13440,y1
	jlt	transf
	move	#>6,x1
	cmp	y1,a #>44102,y1
	jlt	transf
	move	#>7,x1
	cmp	y1,a #>144715,y1
	jlt	transf
	move	#>8,x1
	cmp	y1,a #>474860,y1
	jlt	transf
	move	#>9,x1
	cmp	y1,a #>1558177,y1
	jlt	transf
	move	#>10,x1
	cmp	y1,a #>5112913,y1
	jlt	transf
	move	#>11,x1
	cmp	y1,a
	jlt	transf
	move	#>12,x1		; Maximum

transf
	move	x1,a
	movep	a1,x:STX
	rts
;-----------------------------------------------------------------------------




lastP	= *		; Placemarker where P-code ends
	
;-----------------------------------------------------------------------------
; X and P is aliased in the EVM
; Since this program uses nore than 0000-$01F, we must not allow
; that data memory overlaps with P-memory.
; We can save the most-needed data in the lower on-chips space.
;-----------------------------------------------------------------------------
       org     x:      

data_in_ptr	ds	1		; buffer pointers
lowtoneI_ptr	ds	1
lowtoneQ_ptr	ds	1
hightoneI_ptr	ds	1
hightoneQ_ptr	ds	1
bitfilter_ptr	ds	1
hist_ptr	ds	1

LEFT_sig        ds      1       ; storage for Left Signal
RIGHT_sig       ds      1       ; storage for Left Signal
switches	ds      1       ; save switch status

LowTonePhase	ds	1	; Filter carrier parameters
LowToneIncr	ds	1
HighTonePhase	ds	1
HighToneIncr	ds	1
AFSKPhase	ds	1	; AFSK tone parameters
AFSKIncr	ds	1
Tone1		ds	1	; Tone library
Tone2		ds	1
AFSKOut		ds	1

HiAmpl		ds	1
LoAmpl		ds	1
HiI		ds	1
LoI		ds	1
HiQ		ds	1
LoQ		ds	1

TmpI		ds	1 	; Temporary storage for raw input I/Q
TmpQ		ds	1

dataout		ds	1
decimate	ds	1 	; bit filter decimator	
m_peak		ds	1	; LED peak detectors
s_peak		ds	1
ClaccH		ds	1	; 32-bit timing factor
ClaccL		ds	1
tcount		ds	1	; timer counter

TxState		ds	1	; TxState as received from host
PTTFlag		ds	1	; PTT flag

save_r0		ds	1	; used with timer ISR
save_m0		ds	1
save_a0		ds	1
save_a1		ds	1
save_a2		ds	1
save_x0		ds	1

temp		ds	1

;-----------------------------------------------------------------------------
; Here we have to be careful because these buffers needs a lot of space
; so move it to reside beyond P-code. This is becauser P and X is aliased.
;-----------------------------------------------------------------------------
	org x:lastP
Buffer		dsm	AliasFilterLen	; storage for input data
LowToneFiltI  	dsm 	SymbolLen	; Integrate & dump buffers
LowToneFiltQ  	dsm 	SymbolLen
HighToneFiltI 	dsm 	SymbolLen
HighToneFiltQ 	dsm 	SymbolLen
BitFilter	dsm	BitFilterLen	; storage for bit filter data
history		dsm	htaps		; trace buffer

;-----------------------------------------------------------------------------

	org Y:
AliasFilterInpI ds 	AliasFilterLen      ; Anti-alias input filter (I/Q)
AliasFilterInpQ ds 	AliasFilterLen
RxBitFilter	ds 	BitFilterLen
LowFreq 	equ 	AliasLowFreq/SampleFreq
UppFreq 	equ 	AliasUppFreq/SampleFreq
CFrq 		equ 	(LowFreq+UppFreq)/2
SideFreq  	equ 	(UppFreq-LowFreq)/2

w0      equ     0.35875 ;coeff. for Blackman-Harris 4 term
w1      equ     0.48829 ;minimum sidelobe window (copied from Motorola BBS)
w2      equ     0.14128
w3	equ	0.01168

;---------------------------------------------------------------------------
        org Y:AliasFilterInpI

time = -@cvf(AliasFilterLen/2)+0.5
count   set 0
        dup AliasFilterLen
angle = PI*time/@cvf(AliasFilterLen/2)
Window = w0+w1*@cos(angle)+w2*@cos(2.0*angle)+w3*@cos(3.0*angle)
Filter = @sin(2.0*PI*time*SideFreq)/(PI*time)
        dc @cos(2.0*PI*time*CFrq)*Window*Filter
time = time+1.0
count   set count+1
        endm

;---------------------------------------------------------------------------
        org Y:AliasFilterInpQ

time = -@cvf(AliasFilterLen/2)+0.5
count   set 0
        dup AliasFilterLen
angle = PI*time/@cvf(AliasFilterLen/2)
Window = w0+w1*@cos(angle)+w2*@cos(2.0*angle)+w3*@cos(3.0*angle)
Filter = @sin(2.0*PI*time*SideFreq)/(PI*time)
        dc -@sin(2.0*PI*time*CFrq)*Window*Filter
time = time+1.0
count   set count+1
        endm

;---------------------------------------------------------------------------
	org Y:RxBitFilter

time = -@cvf(BitFilterLen/2)+0.5
count   set 0
	dup BitFilterLen
Filter = 0.75*@cos(PI*time/@cvf(BitFilterLen))+0.25*@cos(3*PI*time/@cvf(BitFilterLen))
time = time+1.0
	dc Filter/(BitFilterLen)*(SampleFreq/2.0/CarDev)/8.0
count   set count+1
	endm
;---------------------------------------------------------------------------
	end
